"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hebe = void 0;
const node_html_parser_1 = require("node-html-parser");
const querystring_1 = require("querystring");
const _1 = require(".");
class Hebe {
    constructor({ username, password } = {}) {
        this.baseUrl = "https://www.hebe.pl";
        /**
         * Obtains `this.token` with optionally specified credentials. If not specified, credentials from constructor are to be used.
         *
         * @param authenticationParams - optional, if specified will override set on contructor `this.username` and `this.password`
         */
        this.authenticate = (...args_1) => __awaiter(this, [...args_1], void 0, function* ({ username, password, } = {}) {
            if (username) {
                this.username = username;
            }
            if (password) {
                this.password = password;
            }
            const { csrfToken, dwsidToken } = yield this.obtainInitialToken();
            this.token = yield this.obtainAuthDwsidToken(dwsidToken, csrfToken);
        });
        /**
         * Retrieves users orders. Authenticates if not authenticated
         *
         * @param OrdersPagination: optional, if not specified, `maxOrders=100` and `start=0`
         * @returns a list of Order class
         */
        this.getOrders = (...args_2) => __awaiter(this, [...args_2], void 0, function* ({ maxOrders, start } = {}) {
            if (!this.token) {
                yield this.authenticate();
            }
            if (!maxOrders) {
                maxOrders = 100;
            }
            if (!start) {
                start = 0;
            }
            const orders = [];
            for (let i = start; i < maxOrders; i += 5) {
                const ordersBatch = yield this.obtainOrders(i);
                if (ordersBatch.length === 0) {
                    break;
                }
                orders.push(...ordersBatch);
            }
            return orders.slice(0, maxOrders);
        });
        /**
         * Retrieves products for a particular order. Authenticates if not authenticated
         *
         * @param order - instance of an Order class to retrieve the products for
         * @returns a list of Product class instances
         */
        this.getProducts = (order) => __awaiter(this, void 0, void 0, function* () {
            if (!this.token) {
                yield this.authenticate();
            }
            const url = `${this.baseUrl}/on/demandware.store/Sites-Hebe-Site/en_US/Order-Orders`;
            const body = (0, querystring_1.stringify)({
                [`dwfrm_orders_orderlist_i${order.position}_show`]: "POKA%C5%BB+SZCZEG%C3%93%C5%81Y",
            });
            const response = yield fetch(url, {
                method: "POST",
                headers: this.headers,
                body,
            });
            if (response.status !== 200) {
                throw new Error(`Product retrieval failed, status code: ${response.status}, url: ${url}`);
            }
            const txt = yield response.text();
            const root = (0, node_html_parser_1.parse)(txt);
            const productsRaw = root.querySelectorAll("div.product-package__row");
            productsRaw.shift();
            return productsRaw.map((productRaw) => new _1.Product(productRaw));
        });
        /**
         * Retrieves all products for by default maximum of 100 orders
         *
         * @param params - Orders pagination params, optional.
         * @returns all products for all orders in the account. By default maximum of 100 orders products
         */
        this.getAllProducts = (...args_3) => __awaiter(this, [...args_3], void 0, function* (params = {}) {
            const orders = yield this.getOrders(params);
            let products = [];
            for (const order of orders) {
                products = products.concat(yield this.getProducts(order));
            }
            return products;
        });
        this.obtainOrders = (...args_4) => __awaiter(this, [...args_4], void 0, function* (start = 0) {
            const url = `${this.baseUrl}/orders?order_status=1&start=${start}`;
            const response = yield fetch(url, {
                headers: this.headers,
            });
            if (response.status !== 200) {
                throw new Error(`Orders retrieval failed, status code: ${response.status}, url: ${url}`);
            }
            const txt = yield response.text();
            if (!txt.includes("orders-detail__section")) {
                return [];
            }
            const root = (0, node_html_parser_1.parse)(txt);
            const ordersDetails = root.querySelectorAll(".orders-detail__section");
            const orders = ordersDetails.map((orderDetails, index) => new _1.Order(orderDetails, start + index));
            return orders;
        });
        this.obtainAuthDwsidToken = (initialDwsidToken, csrfToken) => __awaiter(this, void 0, void 0, function* () {
            const url = `${this.baseUrl}/on/demandware.store/Sites-Hebe-Site/en_US/Login-LoginForm`;
            const headers = Object.assign(Object.assign({}, this.headers), { Cookie: `dwsid=${initialDwsidToken};` });
            const body = (0, querystring_1.stringify)({
                dwfrm_login_username_d0uexrwjfbur: this.username,
                dwfrm_login_password_d0vyhxgwmmwx: this.password,
                csrf_token: csrfToken,
                dwfrm_login_login: "",
                format: "ajax",
            });
            const response = yield fetch(url, {
                method: "POST",
                headers,
                body,
            });
            if (response.status !== 200) {
                throw new Error(`Authentication failed, status code: ${response.status}`);
            }
            const { error, status } = JSON.parse(yield response.text());
            if (error || status !== "SUCCESS") {
                throw new Error(`Authentication failed: ${error}. Status: ${status}`);
            }
            const dwsidCookie = response.headers
                .getSetCookie()
                .find((cookie) => cookie.includes("dwsid"));
            if (!dwsidCookie) {
                throw new Error("dwsid cookie is not present in the initial token retrieval");
            }
            const dwsidToken = dwsidCookie.split("dwsid=")[1].split(";")[0];
            if (!dwsidToken || typeof dwsidToken !== "string") {
                throw new Error("dwsidToken is not present in the initial token retrieval");
            }
            return dwsidToken;
        });
        this.obtainInitialToken = () => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const response = yield fetch(this.baseUrl);
            const dwsidCookie = response.headers
                .getSetCookie()
                .find((cookie) => cookie.includes("dwsid"));
            const txt = yield response.text();
            if (!dwsidCookie) {
                throw new Error("dwsid cookie is not present in the initial token retrieval");
            }
            const dwsidToken = dwsidCookie.split("dwsid=")[1].split(";")[0];
            const csrfRaw = (_a = txt === null || txt === void 0 ? void 0 : txt.split("window.CSRFToken = ")[1]) === null || _a === void 0 ? void 0 : _a.split(";")[0];
            if (!csrfRaw) {
                throw new Error("csrfToken is not present in the initial token retrieval");
            }
            const csrfJson = JSON.parse(csrfRaw);
            const csrfToken = csrfJson === null || csrfJson === void 0 ? void 0 : csrfJson.value;
            if (!csrfToken || typeof csrfToken !== "string") {
                throw new Error("csrfToken is not present in the initial token retrieval");
            }
            return {
                csrfToken,
                dwsidToken,
            };
        });
        this.username = username !== null && username !== void 0 ? username : "";
        this.password = password !== null && password !== void 0 ? password : "";
    }
    get headers() {
        return Object.assign({ "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8", "X-Requested-With": "XMLHttpRequest", Origin: this.baseUrl, Referer: `${this.baseUrl}/home?showform=true&targeturl=%2Faccount` }, (this.token && {
            Cookie: `dwsid=${this.token}`,
        }));
    }
}
exports.Hebe = Hebe;
